"""A library of opinionated enhancements for Textual."""

##############################################################################
# Python imports.
from importlib.metadata import version

######################################################################
# Main app information.
__author__ = "Dave Pearson"
__copyright__ = "Copyright 2025, Dave Pearson"
__credits__ = ["Dave Pearson"]
__maintainer__ = "Dave Pearson"
__email__ = "davep@davep.org"
__version__ = version("textual_enhanced")
__licence__ = "MIT"

### __init__.py ends here
